@@ -57,12 +57,10 @@ array_take(PyArrayObject *self, PyObject *args, PyObject *kwds)
     static char *kwlist[] = {"indices", "axis", "out", "mode", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|O&O&O&", kwlist,
-                                     &indices, PyArray_AxisConverter,
-                                     &dimension,
-                                     PyArray_OutputConverter,
-                                     &out,
-                                     PyArray_ClipmodeConverter,
-                                     &mode))
+                                     &indices,
+                                     PyArray_AxisConverter, &dimension,
+                                     PyArray_OutputConverter, &out,
+                                     PyArray_ClipmodeConverter, &mode))
         return NULL;
 
     return _ARET(PyArray_TakeFrom(self, indices, dimension, out, mode));
@@ -90,9 +88,9 @@ array_put(PyArrayObject *self, PyObject *args, PyObject *kwds)
     static char *kwlist[] = {"indices", "values", "mode", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "OO|O&", kwlist,
-                                     &indices, &values,
-                                     PyArray_ClipmodeConverter,
-                                     &mode))
+                                     &indices,
+                                     &values,
+                                     PyArray_ClipmodeConverter, &mode))
         return NULL;
     return PyArray_PutTo(self, values, indices, mode);
 }
@@ -203,10 +201,8 @@ array_argmax(PyArrayObject *self, PyObject *args, PyObject *kwds)
     static char *kwlist[] = {"axis", "out", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "|O&O&", kwlist,
-                                     PyArray_AxisConverter,
-                                     &axis,
-                                     PyArray_OutputConverter,
-                                     &out))
+                                     PyArray_AxisConverter, &axis,
+                                     PyArray_OutputConverter, &out))
         return NULL;
 
     return _ARET(PyArray_ArgMax(self, axis, out));
@@ -220,10 +216,8 @@ array_argmin(PyArrayObject *self, PyObject *args, PyObject *kwds)
     static char *kwlist[] = {"axis", "out", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "|O&O&", kwlist,
-                                     PyArray_AxisConverter,
-                                     &axis,
-                                     PyArray_OutputConverter,
-                                     &out))
+                                     PyArray_AxisConverter, &axis,
+                                     PyArray_OutputConverter, &out))
         return NULL;
 
     return _ARET(PyArray_ArgMin(self, axis, out));
@@ -237,10 +231,8 @@ array_max(PyArrayObject *self, PyObject *args, PyObject *kwds)
     static char *kwlist[] = {"axis", "out", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "|O&O&", kwlist,
-                                     PyArray_AxisConverter,
-                                     &axis,
-                                     PyArray_OutputConverter,
-                                     &out))
+                                     PyArray_AxisConverter, &axis,
+                                     PyArray_OutputConverter, &out))
         return NULL;
 
     return PyArray_Max(self, axis, out);
@@ -254,10 +246,8 @@ array_ptp(PyArrayObject *self, PyObject *args, PyObject *kwds)
     static char *kwlist[] = {"axis", "out", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "|O&O&", kwlist,
-                                     PyArray_AxisConverter,
-                                     &axis,
-                                     PyArray_OutputConverter,
-                                     &out))
+                                     PyArray_AxisConverter, &axis,
+                                     PyArray_OutputConverter, &out))
         return NULL;
 
     return PyArray_Ptp(self, axis, out);
@@ -272,10 +262,8 @@ array_min(PyArrayObject *self, PyObject *args, PyObject *kwds)
     static char *kwlist[] = {"axis", "out", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "|O&O&", kwlist,
-                                     PyArray_AxisConverter,
-                                     &axis,
-                                     PyArray_OutputConverter,
-                                     &out))
+                                     PyArray_AxisConverter, &axis,
+                                     PyArray_OutputConverter, &out))
         return NULL;
 
     return PyArray_Min(self, axis, out);
@@ -315,7 +303,8 @@ PyArray_GetField(PyArrayObject *self, PyArray_Descr *typed, int offset)
                                self->nd, self->dimensions,
                                self->strides,
                                self->data + offset,
-                               self->flags, (PyObject *)self);
+                               self->flags&(~NPY_F_CONTIGUOUS),
+                               (PyObject *)self);
     if (ret == NULL) {
         return NULL;
     }
@@ -335,8 +324,8 @@ array_getfield(PyArrayObject *self, PyObject *args, PyObject *kwds)
     static char *kwlist[] = {"dtype", "offset", 0};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "O&|i", kwlist,
-                                     PyArray_DescrConverter,
-                                     &dtype, &offset)) {
+                                     PyArray_DescrConverter, &dtype,
+                                     &offset)) {
         Py_XDECREF(dtype);
         return NULL;
     }
@@ -388,12 +377,20 @@ array_setfield(PyArrayObject *self, PyObject *args, PyObject *kwds)
     static char *kwlist[] = {"value", "dtype", "offset", 0};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "OO&|i", kwlist,
-                                     &value, PyArray_DescrConverter,
-                                     &dtype, &offset)) {
+                                     &value,
+                                     PyArray_DescrConverter, &dtype,
+                                     &offset)) {
         Py_XDECREF(dtype);
         return NULL;
     }
 
+    if (PyDataType_REFCHK(PyArray_DESCR(self))) {
+        PyErr_SetString(PyExc_RuntimeError,
+                    "cannot call setfield on an object array");
+        Py_DECREF(dtype);
+        return NULL;
+    }
+
     if (PyArray_SetField(self, dtype, offset, value) < 0) {
         return NULL;
     }
@@ -459,7 +456,8 @@ array_byteswap(PyArrayObject *self, PyObject *args)
 {
     Bool inplace = FALSE;
 
-    if (!PyArg_ParseTuple(args, "|O&", PyArray_BoolConverter, &inplace)) {
+    if (!PyArg_ParseTuple(args, "|O&",
+                            PyArray_BoolConverter, &inplace)) {
         return NULL;
     }
     return PyArray_Byteswap(self, inplace);
@@ -482,8 +480,7 @@ array_tostring(PyArrayObject *self, PyObject *args, PyObject *kwds)
     static char *kwlist[] = {"order", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "|O&", kwlist,
-                                     PyArray_OrderConverter,
-                                     &order)) {
+                                     PyArray_OrderConverter, &order)) {
         return NULL;
     }
     return PyArray_ToString(self, order);
@@ -496,7 +493,7 @@ array_tostring(PyArrayObject *self, PyObject *args, PyObject *kwds)
 static PyObject *
 array_tofile(PyArrayObject *self, PyObject *args, PyObject *kwds)
 {
-    int ret;
+    int ret, ret2;
     PyObject *file;
     FILE *fd;
     char *sep = "";
@@ -504,7 +501,9 @@ array_tofile(PyArrayObject *self, PyObject *args, PyObject *kwds)
     static char *kwlist[] = {"file", "sep", "format", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|ss", kwlist,
-                                     &file, &sep, &format)) {
+                                     &file,
+                                     &sep,
+                                     &format)) {
         return NULL;
     }
 
@@ -517,11 +516,7 @@ array_tofile(PyArrayObject *self, PyObject *args, PyObject *kwds)
     else {
         Py_INCREF(file);
     }
-#if defined(NPY_PY3K)
     fd = npy_PyFile_Dup(file, "wb");
-#else
-    fd = PyFile_AsFile(file);
-#endif
     if (fd == NULL) {
         PyErr_SetString(PyExc_IOError, "first argument must be a " \
                         "string or open file");
@@ -529,11 +524,9 @@ array_tofile(PyArrayObject *self, PyObject *args, PyObject *kwds)
         return NULL;
     }
     ret = PyArray_ToFile(self, fd, sep, format);
-#if defined(NPY_PY3K)
-    fclose(fd);
-#endif
+    ret2 = npy_PyFile_DupClose(file, fd);
     Py_DECREF(file);
-    if (ret < 0) {
+    if (ret < 0 || ret2 < 0) {
         return NULL;
     }
     Py_INCREF(Py_None);
@@ -750,12 +743,35 @@ array_setscalar(PyArrayObject *self, PyObject *args) {
     return Py_None;
 }
 
+/* Sets the array values from another array as if they were flat */
+static PyObject *
+array_setasflat(PyArrayObject *self, PyObject *args)
+{
+    PyObject *arr_in;
+    PyArrayObject *arr;
+
+    if (!PyArg_ParseTuple(args, "O", &arr_in)) {
+        return NULL;
+    }
+
+    arr = (PyArrayObject *)PyArray_FromAny(arr_in, NULL, 0, 0, 0, NULL);
+    if (arr == NULL) {
+        return NULL;
+    }
+
+    if (PyArray_CopyAnyInto(self, arr) != 0) {
+        Py_DECREF(arr);
+        return NULL;
+    }
+
+    Py_DECREF(arr);
+    Py_RETURN_NONE;
+}
 
 static PyObject *
-array_cast(PyArrayObject *self, PyObject *args)
+array_astype(PyArrayObject *self, PyObject *args)
 {
     PyArray_Descr *descr = NULL;
-    PyObject *obj;
 
     if (!PyArg_ParseTuple(args, "O&", PyArray_DescrConverter,
                           &descr)) {
@@ -763,19 +779,6 @@ array_cast(PyArrayObject *self, PyObject *args)
         return NULL;
     }
 
-    if (PyArray_EquivTypes(descr, self->descr)) {
-        obj = _ARET(PyArray_NewCopy(self,NPY_ANYORDER));
-        Py_XDECREF(descr);
-        return obj;
-    }
-    if (descr->names != NULL) {
-        int flags;
-        flags = NPY_FORCECAST;
-        if (PyArray_ISFORTRAN(self)) {
-            flags |= NPY_FORTRAN;
-        }
-        return PyArray_FromArray(self, descr, flags);
-    }
     return PyArray_CastToType(self, descr, PyArray_ISFORTRAN(self));
 }
 
@@ -871,8 +874,8 @@ array_getarray(PyArrayObject *self, PyObject *args)
     PyArray_Descr *newtype = NULL;
     PyObject *ret;
 
-    if (!PyArg_ParseTuple(args, "|O&", PyArray_DescrConverter,
-                          &newtype)) {
+    if (!PyArg_ParseTuple(args, "|O&",
+                            PyArray_DescrConverter, &newtype)) {
         Py_XDECREF(newtype);
         return NULL;
     }
@@ -918,15 +921,17 @@ array_getarray(PyArrayObject *self, PyObject *args)
 
 
 static PyObject *
-array_copy(PyArrayObject *self, PyObject *args)
+array_copy(PyArrayObject *self, PyObject *args, PyObject *kwds)
 {
-    PyArray_ORDER fortran=PyArray_CORDER;
-    if (!PyArg_ParseTuple(args, "|O&", PyArray_OrderConverter,
-                          &fortran)) {
+    PyArray_ORDER order = PyArray_CORDER;
+    static char *kwlist[] = {"order", NULL};
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "|O&", kwlist,
+                            PyArray_OrderConverter, &order)) {
         return NULL;
     }
 
-    return PyArray_NewCopy(self, fortran);
+    return PyArray_NewCopy(self, order);
 }
 
 #include <stdio.h>
@@ -980,8 +985,8 @@ array_repeat(PyArrayObject *self, PyObject *args, PyObject *kwds) {
     static char *kwlist[] = {"repeats", "axis", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|O&", kwlist,
-                                     &repeats, PyArray_AxisConverter,
-                                     &axis)) {
+                                     &repeats,
+                                     PyArray_AxisConverter, &axis)) {
         return NULL;
     }
     return _ARET(PyArray_Repeat(self, repeats, axis));
@@ -1019,15 +1024,16 @@ array_sort(PyArrayObject *self, PyObject *args, PyObject *kwds)
 {
     int axis=-1;
     int val;
-    PyArray_SORTKIND which = PyArray_QUICKSORT;
+    PyArray_SORTKIND sortkind = PyArray_QUICKSORT;
     PyObject *order = NULL;
     PyArray_Descr *saved = NULL;
     PyArray_Descr *newd;
     static char *kwlist[] = {"axis", "kind", "order", NULL};
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwds, "|iO&O", kwlist, &axis,
-                                     PyArray_SortkindConverter, &which,
-                                     &order)) {
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "|iO&O", kwlist,
+                                    &axis,
+                                    PyArray_SortkindConverter, &sortkind,
+                                    &order)) {
         return NULL;
     }
     if (order == Py_None) {
@@ -1057,7 +1063,7 @@ array_sort(PyArrayObject *self, PyObject *args, PyObject *kwds)
         self->descr = newd;
     }
 
-    val = PyArray_Sort(self, axis, which);
+    val = PyArray_Sort(self, axis, sortkind);
     if (order != NULL) {
         Py_XDECREF(self->descr);
         self->descr = saved;
@@ -1073,14 +1079,14 @@ static PyObject *
 array_argsort(PyArrayObject *self, PyObject *args, PyObject *kwds)
 {
     int axis = -1;
-    PyArray_SORTKIND which = PyArray_QUICKSORT;
+    PyArray_SORTKIND sortkind = PyArray_QUICKSORT;
     PyObject *order = NULL, *res;
     PyArray_Descr *newd, *saved=NULL;
     static char *kwlist[] = {"axis", "kind", "order", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "|O&O&O", kwlist,
                                      PyArray_AxisConverter, &axis,
-                                     PyArray_SortkindConverter, &which,
+                                     PyArray_SortkindConverter, &sortkind,
                                      &order)) {
         return NULL;
     }
@@ -1111,7 +1117,7 @@ array_argsort(PyArrayObject *self, PyObject *args, PyObject *kwds)
         self->descr = newd;
     }
 
-    res = PyArray_ArgSort(self, axis, which);
+    res = PyArray_ArgSort(self, axis, sortkind);
     if (order != NULL) {
         Py_XDECREF(self->descr);
         self->descr = saved;
@@ -1359,14 +1365,19 @@ array_setstate(PyArrayObject *self, PyObject *args)
     /* This will free any memory associated with a and
        use the string in setstate as the (writeable) memory.
     */
-    if (!PyArg_ParseTuple(args, "(iO!O!iO)", &version, &PyTuple_Type,
-                          &shape, &PyArrayDescr_Type, &typecode,
-                          &fortran, &rawdata)) {
+    if (!PyArg_ParseTuple(args, "(iO!O!iO)",
+                            &version,
+                            &PyTuple_Type, &shape,
+                            &PyArrayDescr_Type, &typecode,
+                            &fortran,
+                            &rawdata)) {
         PyErr_Clear();
         version = 0;
-        if (!PyArg_ParseTuple(args, "(O!O!iO)", &PyTuple_Type,
-                              &shape, &PyArrayDescr_Type, &typecode,
-                              &fortran, &rawdata)) {
+        if (!PyArg_ParseTuple(args, "(O!O!iO)",
+                            &PyTuple_Type, &shape,
+                            &PyArrayDescr_Type, &typecode,
+                            &fortran,
+                            &rawdata)) {
             return NULL;
         }
     }
@@ -1679,11 +1690,9 @@ array_mean(PyArrayObject *self, PyObject *args, PyObject *kwds)
     static char *kwlist[] = {"axis", "dtype", "out", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "|O&O&O&", kwlist,
-                                     PyArray_AxisConverter,
-                                     &axis, PyArray_DescrConverter2,
-                                     &dtype,
-                                     PyArray_OutputConverter,
-                                     &out)) {
+                                     PyArray_AxisConverter, &axis,
+                                     PyArray_DescrConverter2, &dtype,
+                                     PyArray_OutputConverter, &out)) {
         Py_XDECREF(dtype);
         return NULL;
     }
@@ -1703,11 +1712,9 @@ array_sum(PyArrayObject *self, PyObject *args, PyObject *kwds)
     static char *kwlist[] = {"axis", "dtype", "out", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "|O&O&O&", kwlist,
-                                     PyArray_AxisConverter,
-                                     &axis, PyArray_DescrConverter2,
-                                     &dtype,
-                                     PyArray_OutputConverter,
-                                     &out)) {
+                                     PyArray_AxisConverter, &axis,
+                                     PyArray_DescrConverter2, &dtype,
+                                     PyArray_OutputConverter, &out)) {
         Py_XDECREF(dtype);
         return NULL;
     }
@@ -1728,11 +1735,9 @@ array_cumsum(PyArrayObject *self, PyObject *args, PyObject *kwds)
     static char *kwlist[] = {"axis", "dtype", "out", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "|O&O&O&", kwlist,
-                                     PyArray_AxisConverter,
-                                     &axis, PyArray_DescrConverter2,
-                                     &dtype,
-                                     PyArray_OutputConverter,
-                                     &out)) {
+                                     PyArray_AxisConverter, &axis,
+                                     PyArray_DescrConverter2, &dtype,
+                                     PyArray_OutputConverter, &out)) {
         Py_XDECREF(dtype);
         return NULL;
     }
@@ -1752,11 +1757,9 @@ array_prod(PyArrayObject *self, PyObject *args, PyObject *kwds)
     static char *kwlist[] = {"axis", "dtype", "out", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "|O&O&O&", kwlist,
-                                     PyArray_AxisConverter,
-                                     &axis, PyArray_DescrConverter2,
-                                     &dtype,
-                                     PyArray_OutputConverter,
-                                     &out)) {
+                                     PyArray_AxisConverter, &axis,
+                                     PyArray_DescrConverter2, &dtype,
+                                     PyArray_OutputConverter, &out)) {
         Py_XDECREF(dtype);
         return NULL;
     }
@@ -1776,11 +1779,9 @@ array_cumprod(PyArrayObject *self, PyObject *args, PyObject *kwds)
     static char *kwlist[] = {"axis", "dtype", "out", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "|O&O&O&", kwlist,
-                                     PyArray_AxisConverter,
-                                     &axis, PyArray_DescrConverter2,
-                                     &dtype,
-                                     PyArray_OutputConverter,
-                                     &out)) {
+                                     PyArray_AxisConverter, &axis,
+                                     PyArray_DescrConverter2, &dtype,
+                                     PyArray_OutputConverter, &out)) {
         Py_XDECREF(dtype);
         return NULL;
     }
@@ -1822,10 +1823,8 @@ array_any(PyArrayObject *self, PyObject *args, PyObject *kwds)
     static char *kwlist[] = {"axis", "out", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "|O&O&", kwlist,
-                                     PyArray_AxisConverter,
-                                     &axis,
-                                     PyArray_OutputConverter,
-                                     &out))
+                                     PyArray_AxisConverter, &axis,
+                                     PyArray_OutputConverter, &out))
         return NULL;
 
     return PyArray_Any(self, axis, out);
@@ -1840,10 +1839,8 @@ array_all(PyArrayObject *self, PyObject *args, PyObject *kwds)
     static char *kwlist[] = {"axis", "out", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "|O&O&", kwlist,
-                                     PyArray_AxisConverter,
-                                     &axis,
-                                     PyArray_OutputConverter,
-                                     &out))
+                                     PyArray_AxisConverter, &axis,
+                                     PyArray_OutputConverter, &out))
         return NULL;
 
     return PyArray_All(self, axis, out);
@@ -1861,11 +1858,10 @@ array_stddev(PyArrayObject *self, PyObject *args, PyObject *kwds)
     static char *kwlist[] = {"axis", "dtype", "out", "ddof", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "|O&O&O&i", kwlist,
-                                     PyArray_AxisConverter,
-                                     &axis, PyArray_DescrConverter2,
-                                     &dtype,
-                                     PyArray_OutputConverter,
-                                     &out, &ddof)) {
+                                     PyArray_AxisConverter, &axis,
+                                     PyArray_DescrConverter2, &dtype,
+                                     PyArray_OutputConverter, &out,
+                                     &ddof)) {
         Py_XDECREF(dtype);
         return NULL;
     }
@@ -1887,11 +1883,10 @@ array_variance(PyArrayObject *self, PyObject *args, PyObject *kwds)
     static char *kwlist[] = {"axis", "dtype", "out", "ddof", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "|O&O&O&i", kwlist,
-                                     PyArray_AxisConverter,
-                                     &axis, PyArray_DescrConverter2,
-                                     &dtype,
-                                     PyArray_OutputConverter,
-                                     &out, &ddof)) {
+                                     PyArray_AxisConverter, &axis,
+                                     PyArray_DescrConverter2, &dtype,
+                                     PyArray_OutputConverter, &out,
+                                     &ddof)) {
         Py_XDECREF(dtype);
         return NULL;
     }
@@ -1911,10 +1906,9 @@ array_compress(PyArrayObject *self, PyObject *args, PyObject *kwds)
     static char *kwlist[] = {"condition", "axis", "out", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|O&O&", kwlist,
-                                     &condition, PyArray_AxisConverter,
-                                     &axis,
-                                     PyArray_OutputConverter,
-                                     &out)) {
+                                     &condition,
+                                     PyArray_AxisConverter, &axis,
+                                     PyArray_OutputConverter, &out)) {
         return NULL;
     }
     return _ARET(PyArray_Compress(self, condition, axis, out));
@@ -1941,7 +1935,9 @@ array_trace(PyArrayObject *self, PyObject *args, PyObject *kwds)
     static char *kwlist[] = {"offset", "axis1", "axis2", "dtype", "out", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "|iiiO&O&", kwlist,
-                                     &offset, &axis1, &axis2,
+                                     &offset,
+                                     &axis1,
+                                     &axis2,
                                      PyArray_DescrConverter2, &dtype,
                                      PyArray_OutputConverter, &out)) {
         Py_XDECREF(dtype);
@@ -1964,9 +1960,9 @@ array_clip(PyArrayObject *self, PyObject *args, PyObject *kwds)
     static char *kwlist[] = {"min", "max", "out", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "|OOO&", kwlist,
-                                     &min, &max,
-                                     PyArray_OutputConverter,
-                                     &out)) {
+                                     &min,
+                                     &max,
+                                     PyArray_OutputConverter, &out)) {
         return NULL;
     }
     if (max == NULL && min == NULL) {
@@ -1998,7 +1994,9 @@ array_diagonal(PyArrayObject *self, PyObject *args, PyObject *kwds)
     static char *kwlist[] = {"offset", "axis1", "axis2", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "|iii", kwlist,
-                                     &offset, &axis1, &axis2)) {
+                                     &offset,
+                                     &axis1,
+                                     &axis2)) {
         return NULL;
     }
     return _ARET(PyArray_Diagonal(self, offset, axis1, axis2));
@@ -2006,27 +2004,30 @@ array_diagonal(PyArrayObject *self, PyObject *args, PyObject *kwds)
 
 
 static PyObject *
-array_flatten(PyArrayObject *self, PyObject *args)
+array_flatten(PyArrayObject *self, PyObject *args, PyObject *kwds)
 {
-    PyArray_ORDER fortran = PyArray_CORDER;
+    PyArray_ORDER order = PyArray_CORDER;
+    static char *kwlist[] = {"order", NULL};
 
-    if (!PyArg_ParseTuple(args, "|O&", PyArray_OrderConverter, &fortran)) {
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "|O&", kwlist,
+                            PyArray_OrderConverter, &order)) {
         return NULL;
     }
-    return PyArray_Flatten(self, fortran);
+    return PyArray_Flatten(self, order);
 }
 
 
 static PyObject *
-array_ravel(PyArrayObject *self, PyObject *args)
+array_ravel(PyArrayObject *self, PyObject *args, PyObject *kwds)
 {
-    PyArray_ORDER fortran = PyArray_CORDER;
+    PyArray_ORDER order = PyArray_CORDER;
+    static char *kwlist[] = {"order", NULL};
 
-    if (!PyArg_ParseTuple(args, "|O&", PyArray_OrderConverter,
-                          &fortran)) {
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "|O&", kwlist,
+                            PyArray_OrderConverter, &order)) {
         return NULL;
     }
-    return PyArray_Ravel(self, fortran);
+    return PyArray_Ravel(self, order);
 }
 
 
@@ -2038,8 +2039,8 @@ array_round(PyArrayObject *self, PyObject *args, PyObject *kwds)
     static char *kwlist[] = {"decimals", "out", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "|iO&", kwlist,
-                                     &decimals, PyArray_OutputConverter,
-                                     &out)) {
+                                     &decimals,
+                                     PyArray_OutputConverter, &out)) {
         return NULL;
     }
     return _ARET(PyArray_Round(self, decimals, out));
@@ -2057,7 +2058,9 @@ array_setflags(PyArrayObject *self, PyObject *args, PyObject *kwds)
     int flagback = self->flags;
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "|OOO", kwlist,
-                                     &write, &align, &uic))
+                                     &write,
+                                     &align,
+                                     &uic))
         return NULL;
 
     if (align != Py_None) {
@@ -2146,7 +2149,7 @@ NPY_NO_EXPORT PyMethodDef array_methods[] = {
     /* for the copy module */
     {"__copy__",
         (PyCFunction)array_copy,
-        METH_VARARGS, NULL},
+        METH_VARARGS | METH_KEYWORDS, NULL},
     {"__deepcopy__",
         (PyCFunction)array_deepcopy,
         METH_VARARGS, NULL},
@@ -2182,7 +2185,7 @@ NPY_NO_EXPORT PyMethodDef array_methods[] = {
         (PyCFunction)array_argsort,
         METH_VARARGS | METH_KEYWORDS, NULL},
     {"astype",
-        (PyCFunction)array_cast,
+        (PyCFunction)array_astype,
         METH_VARARGS, NULL},
     {"byteswap",
         (PyCFunction)array_byteswap,
@@ -2204,7 +2207,7 @@ NPY_NO_EXPORT PyMethodDef array_methods[] = {
         METH_VARARGS, NULL},
     {"copy",
         (PyCFunction)array_copy,
-        METH_VARARGS, NULL},
+        METH_VARARGS | METH_KEYWORDS, NULL},
     {"cumprod",
         (PyCFunction)array_cumprod,
         METH_VARARGS | METH_KEYWORDS, NULL},
@@ -2222,7 +2225,7 @@ NPY_NO_EXPORT PyMethodDef array_methods[] = {
         METH_VARARGS, NULL},
     {"flatten",
         (PyCFunction)array_flatten,
-        METH_VARARGS, NULL},
+        METH_VARARGS | METH_KEYWORDS, NULL},
     {"getfield",
         (PyCFunction)array_getfield,
         METH_VARARGS | METH_KEYWORDS, NULL},
@@ -2232,6 +2235,9 @@ NPY_NO_EXPORT PyMethodDef array_methods[] = {
     {"itemset",
         (PyCFunction) array_setscalar,
         METH_VARARGS, NULL},
+    {"setasflat",
+        (PyCFunction) array_setasflat,
+        METH_VARARGS, NULL},
     {"max",
         (PyCFunction)array_max,
         METH_VARARGS | METH_KEYWORDS, NULL},
@@ -2258,7 +2264,7 @@ NPY_NO_EXPORT PyMethodDef array_methods[] = {
         METH_VARARGS | METH_KEYWORDS, NULL},
     {"ravel",
         (PyCFunction)array_ravel,
-        METH_VARARGS, NULL},
+        METH_VARARGS | METH_KEYWORDS, NULL},
     {"repeat",
         (PyCFunction)array_repeat,
         METH_VARARGS | METH_KEYWORDS, NULL},
