@@ -1,12 +1,15 @@
 from StringIO import StringIO
 import pickle
 import sys
+import platform
 import gc
 import copy
 from os import path
 from numpy.testing import *
-from numpy.testing.utils import _assert_valid_refcount
+from numpy.testing.utils import _assert_valid_refcount, WarningManager
 from numpy.compat import asbytes, asunicode, asbytes_nested
+import warnings
+import tempfile
 import numpy as np
 
 if sys.version_info[0] >= 3:
@@ -113,11 +116,11 @@ class TestRegression(TestCase):
         """Ticket #72"""
         a = np.array(['test', 'auto'])
         assert_array_equal(a == 'auto', np.array([False,True]))
-        self.assert_(a[1] == 'auto')
-        self.assert_(a[0] != 'auto')
+        self.assertTrue(a[1] == 'auto')
+        self.assertTrue(a[0] != 'auto')
         b = np.linspace(0, 10, 11)
-        self.assert_(b != 'auto')
-        self.assert_(b[0] != 'auto')
+        self.assertTrue(b != 'auto')
+        self.assertTrue(b[0] != 'auto')
 
     def test_unicode_swapping(self,level=rlevel):
         """Ticket #79"""
@@ -136,8 +139,8 @@ class TestRegression(TestCase):
         self.assertRaises(TypeError,np.dtype,
                               {'names':['a'],'formats':['foo']},align=1)
 
-    @dec.knownfailureif(sys.version_info[0] >= 3 or (
-                        sys.platform == 'win32' and '64 bit' in sys.version),
+    @dec.knownfailureif((sys.version_info[0] >= 3) or
+                        (sys.platform == "win32" and platform.architecture()[0] == "64bit"),
                         "numpy.intp('0xff', 16) not supported on Py3, "
                         "as it does not inherit from Python int")
     def test_intp(self,level=rlevel):
@@ -159,8 +162,8 @@ class TestRegression(TestCase):
         yb = ((b>2) & (b<6))
         assert_array_almost_equal(xa,ya.nonzero())
         assert_array_almost_equal(xb,yb.nonzero())
-        assert(np.all(a[ya] > 0.5))
-        assert(np.all(b[yb] > 0.5))
+        assert_(np.all(a[ya] > 0.5))
+        assert_(np.all(b[yb] > 0.5))
 
     def test_mem_dot(self,level=rlevel):
         """Ticket #106"""
@@ -182,7 +185,7 @@ class TestRegression(TestCase):
 #        """Ticket #112"""
 #        if np.longfloat(0).itemsize > 8:
 #            a = np.exp(np.array([1000],dtype=np.longfloat))
-#            assert(str(a)[1:9] == str(a[0])[:8])
+#            assert_(str(a)[1:9] == str(a[0])[:8])
 
     def test_argmax(self,level=rlevel):
         """Ticket #119"""
@@ -206,8 +209,8 @@ class TestRegression(TestCase):
         """Ticket #133"""
         a = np.array([3])
         b = np.array(3)
-        assert(type(a.squeeze()) is np.ndarray)
-        assert(type(b.squeeze()) is np.ndarray)
+        assert_(type(a.squeeze()) is np.ndarray)
+        assert_(type(b.squeeze()) is np.ndarray)
 
     def test_add_identity(self,level=rlevel):
         """Ticket #143"""
@@ -346,7 +349,7 @@ class TestRegression(TestCase):
         dt = np.dtype([('one', '<i4'),('two', '<i4')])
         x = np.array((1,2), dtype=dt)
         x = x.byteswap()
-        assert(x['one'] > 1 and x['two'] > 2)
+        assert_(x['one'] > 1 and x['two'] > 2)
 
     def test_method_args(self, level=rlevel):
         # Make sure methods and functions have same default axis
@@ -487,7 +490,7 @@ class TestRegression(TestCase):
                   np.rec.array([(1,2),(3,4)]),
                   np.rec.fromarrays([(1,2),(3,4)],"i4,i4"),
                   np.rec.fromarrays([(1,2),(3,4)])]:
-            assert(a.dtype in [dt0,dt1])
+            assert_(a.dtype in [dt0,dt1])
 
     def test_random_shuffle(self, level=rlevel):
         """Ticket #374"""
@@ -607,6 +610,9 @@ class TestRegression(TestCase):
         assert_equal(np.dot(x,z),np.dot(x,y2))
 
     def test_object_casting(self, level=rlevel):
+        # This used to trigger the object-type version of
+        # the bitwise_or operation, because float64 -> object
+        # casting succeeds
         def rs():
             x = np.ones([484,286])
             y = np.zeros([484,286])
@@ -719,15 +725,15 @@ class TestRegression(TestCase):
         arr = np.zeros(5, dtype=np.object_)
 
         arr[:] = a
-        assert cnt(a) == cnt0_a + 5
+        assert_equal(cnt(a), cnt0_a + 5)
 
         arr[:] = b
-        assert cnt(a) == cnt0_a
-        assert cnt(b) == cnt0_b + 5
+        assert_equal(cnt(a), cnt0_a)
+        assert_equal(cnt(b), cnt0_b + 5)
 
         arr[:2] = c
-        assert cnt(b) == cnt0_b + 3
-        assert cnt(c) == cnt0_c + 2
+        assert_equal(cnt(b), cnt0_b + 3)
+        assert_equal(cnt(c), cnt0_c + 2)
 
         del arr
 
@@ -911,7 +917,6 @@ class TestRegression(TestCase):
     def test_attributes(self, level=rlevel):
         """Ticket #791
         """
-        import numpy as np
         class TestArray(np.ndarray):
             def __new__(cls, data, info):
                 result = np.array(data)
@@ -921,63 +926,63 @@ class TestRegression(TestCase):
             def __array_finalize__(self, obj):
                 self.info = getattr(obj, 'info', '')
         dat = TestArray([[1,2,3,4],[5,6,7,8]],'jubba')
-        assert dat.info == 'jubba'
+        assert_(dat.info == 'jubba')
         dat.resize((4,2))
-        assert dat.info == 'jubba'
+        assert_(dat.info == 'jubba')
         dat.sort()
-        assert dat.info == 'jubba'
+        assert_(dat.info == 'jubba')
         dat.fill(2)
-        assert dat.info == 'jubba'
+        assert_(dat.info == 'jubba')
         dat.put([2,3,4],[6,3,4])
-        assert dat.info == 'jubba'
+        assert_(dat.info == 'jubba')
         dat.setfield(4, np.int32,0)
-        assert dat.info == 'jubba'
+        assert_(dat.info == 'jubba')
         dat.setflags()
-        assert dat.info == 'jubba'
-        assert dat.all(1).info == 'jubba'
-        assert dat.any(1).info == 'jubba'
-        assert dat.argmax(1).info == 'jubba'
-        assert dat.argmin(1).info == 'jubba'
-        assert dat.argsort(1).info == 'jubba'
-        assert dat.astype(TestArray).info == 'jubba'
-        assert dat.byteswap().info == 'jubba'
-        assert dat.clip(2,7).info == 'jubba'
-        assert dat.compress([0,1,1]).info == 'jubba'
-        assert dat.conj().info == 'jubba'
-        assert dat.conjugate().info == 'jubba'
-        assert dat.copy().info == 'jubba'
+        assert_(dat.info == 'jubba')
+        assert_(dat.all(1).info == 'jubba')
+        assert_(dat.any(1).info == 'jubba')
+        assert_(dat.argmax(1).info == 'jubba')
+        assert_(dat.argmin(1).info == 'jubba')
+        assert_(dat.argsort(1).info == 'jubba')
+        assert_(dat.astype(TestArray).info == 'jubba')
+        assert_(dat.byteswap().info == 'jubba')
+        assert_(dat.clip(2,7).info == 'jubba')
+        assert_(dat.compress([0,1,1]).info == 'jubba')
+        assert_(dat.conj().info == 'jubba')
+        assert_(dat.conjugate().info == 'jubba')
+        assert_(dat.copy().info == 'jubba')
         dat2 = TestArray([2, 3, 1, 0],'jubba')
         choices = [[0, 1, 2, 3], [10, 11, 12, 13],
                    [20, 21, 22, 23], [30, 31, 32, 33]]
-        assert dat2.choose(choices).info == 'jubba'
-        assert dat.cumprod(1).info == 'jubba'
-        assert dat.cumsum(1).info == 'jubba'
-        assert dat.diagonal().info == 'jubba'
-        assert dat.flatten().info == 'jubba'
-        assert dat.getfield(np.int32,0).info == 'jubba'
-        assert dat.imag.info == 'jubba'
-        assert dat.max(1).info == 'jubba'
-        assert dat.mean(1).info == 'jubba'
-        assert dat.min(1).info == 'jubba'
-        assert dat.newbyteorder().info == 'jubba'
-        assert dat.nonzero()[0].info == 'jubba'
-        assert dat.nonzero()[1].info == 'jubba'
-        assert dat.prod(1).info == 'jubba'
-        assert dat.ptp(1).info == 'jubba'
-        assert dat.ravel().info == 'jubba'
-        assert dat.real.info == 'jubba'
-        assert dat.repeat(2).info == 'jubba'
-        assert dat.reshape((2,4)).info == 'jubba'
-        assert dat.round().info == 'jubba'
-        assert dat.squeeze().info == 'jubba'
-        assert dat.std(1).info == 'jubba'
-        assert dat.sum(1).info == 'jubba'
-        assert dat.swapaxes(0,1).info == 'jubba'
-        assert dat.take([2,3,5]).info == 'jubba'
-        assert dat.transpose().info == 'jubba'
-        assert dat.T.info == 'jubba'
-        assert dat.var(1).info == 'jubba'
-        assert dat.view(TestArray).info == 'jubba'
+        assert_(dat2.choose(choices).info == 'jubba')
+        assert_(dat.cumprod(1).info == 'jubba')
+        assert_(dat.cumsum(1).info == 'jubba')
+        assert_(dat.diagonal().info == 'jubba')
+        assert_(dat.flatten().info == 'jubba')
+        assert_(dat.getfield(np.int32,0).info == 'jubba')
+        assert_(dat.imag.info == 'jubba')
+        assert_(dat.max(1).info == 'jubba')
+        assert_(dat.mean(1).info == 'jubba')
+        assert_(dat.min(1).info == 'jubba')
+        assert_(dat.newbyteorder().info == 'jubba')
+        assert_(dat.nonzero()[0].info == 'jubba')
+        assert_(dat.nonzero()[1].info == 'jubba')
+        assert_(dat.prod(1).info == 'jubba')
+        assert_(dat.ptp(1).info == 'jubba')
+        assert_(dat.ravel().info == 'jubba')
+        assert_(dat.real.info == 'jubba')
+        assert_(dat.repeat(2).info == 'jubba')
+        assert_(dat.reshape((2,4)).info == 'jubba')
+        assert_(dat.round().info == 'jubba')
+        assert_(dat.squeeze().info == 'jubba')
+        assert_(dat.std(1).info == 'jubba')
+        assert_(dat.sum(1).info == 'jubba')
+        assert_(dat.swapaxes(0,1).info == 'jubba')
+        assert_(dat.take([2,3,5]).info == 'jubba')
+        assert_(dat.transpose().info == 'jubba')
+        assert_(dat.T.info == 'jubba')
+        assert_(dat.var(1).info == 'jubba')
+        assert_(dat.view(TestArray).info == 'jubba')
 
     def test_recarray_tolist(self, level=rlevel):
         """Ticket #793, changeset r5215
@@ -987,8 +992,8 @@ class TestRegression(TestCase):
         buf = np.zeros(40, dtype=np.int8)
         a = np.recarray(2, formats="i4,f8,f8", names="id,x,y", buf=buf)
         b = a.tolist()
-        assert( a[0].tolist() == b[0])
-        assert( a[1].tolist() == b[1])
+        assert_( a[0].tolist() == b[0])
+        assert_( a[1].tolist() == b[1])
 
     def test_char_array_creation(self, level=rlevel):
         a = np.array('123', dtype='c')
@@ -1009,11 +1014,7 @@ class TestRegression(TestCase):
     def test_sign_for_complex_nan(self, level=rlevel):
         """Ticket 794."""
         C = np.array([-np.inf, -2+1j, 0, 2-1j, np.inf, np.nan])
-        err = np.seterr(invalid="ignore")
-        try:
-            have = np.sign(C)
-        finally:
-            np.seterr(**err)        
+        have = np.sign(C)
         want = np.array([-1+0j, -1+0j, 0+0j, 1+0j, 1+0j, np.nan])
         assert_equal(have, want)
 
@@ -1132,22 +1133,28 @@ class TestRegression(TestCase):
     def test_array_from_sequence_scalar_array(self):
         """Ticket #1078: segfaults when creating an array with a sequence of 0d
         arrays."""
-        a = np.ones(2)
-        b = np.array(3)
-        assert_raises(ValueError, lambda: np.array((a, b)))
+        a = np.array((np.ones(2), np.array(2)))
+        assert_equal(a.shape, (2,))
+        assert_equal(a.dtype, np.dtype(object))
+        assert_equal(a[0], np.ones(2))
+        assert_equal(a[1], np.array(2))
 
-        t = ((1,), np.array(1))
-        assert_raises(ValueError, lambda: np.array(t))
+        a = np.array(((1,), np.array(1)))
+        assert_equal(a.shape, (2,))
+        assert_equal(a.dtype, np.dtype(object))
+        assert_equal(a[0], (1,))
+        assert_equal(a[1], np.array(1))
 
-    @dec.knownfailureif(True, "Fix this for 1.5.0.")
     def test_array_from_sequence_scalar_array2(self):
         """Ticket #1081: weird array with strange input..."""
         t = np.array([np.array([]), np.array(0, object)])
-        assert_raises(ValueError, lambda: np.array(t))
+        assert_equal(t.shape, (2,))
+        assert_equal(t.dtype, np.dtype(object))
 
     def test_array_too_big(self):
         """Ticket #1080."""
-        assert_raises(ValueError, np.zeros, [2**10]*10)
+        assert_raises(ValueError, np.zeros, [975]*7, np.int8)
+        assert_raises(ValueError, np.zeros, [26244]*5, np.int8)
 
     def test_dtype_keyerrors_(self):
         """Ticket #1106."""
@@ -1265,7 +1272,7 @@ class TestRegression(TestCase):
         """Ticket #1317."""
         def test_type(t):
             min = np.array([np.iinfo(t).min])
-            min /= -1
+            min //= -1
 
         old_err = np.seterr(divide="ignore")
         try:
@@ -1309,7 +1316,8 @@ class TestRegression(TestCase):
 
     def test_ticket_1539(self):
         dtypes = [x for x in np.typeDict.values()
-                  if issubclass(x, np.number)]
+                  if (issubclass(x, np.number)
+                      and not issubclass(x, np.timeinteger))]
         a = np.array([], dtypes[0])
         failures = []
         for x in dtypes:
@@ -1377,5 +1385,231 @@ class TestRegression(TestCase):
         c2 = sys.getrefcount(rgba)
         assert_equal(c1, c2)
 
+    def test_fromfile_tofile_seeks(self):
+        # On Python 3, tofile/fromfile used to get (#1610) the Python
+        # file handle out of sync
+        f0 = tempfile.NamedTemporaryFile()
+        f = f0.file
+        f.write(np.arange(255, dtype='u1').tostring())
+
+        f.seek(20)
+        ret = np.fromfile(f, count=4, dtype='u1')
+        assert_equal(ret, np.array([20, 21, 22, 23], dtype='u1'))
+        assert_equal(f.tell(), 24)
+
+        f.seek(40)
+        np.array([1, 2, 3], dtype='u1').tofile(f)
+        assert_equal(f.tell(), 43)
+
+        f.seek(40)
+        data = f.read(3)
+        assert_equal(data, asbytes("\x01\x02\x03"))
+
+        f.seek(80)
+        f.read(4)
+        data = np.fromfile(f, dtype='u1', count=4)
+        assert_equal(data, np.array([84, 85, 86, 87], dtype='u1'))
+
+        f.close()
+
+    def test_complex_scalar_warning(self):
+        for tp in [np.csingle, np.cdouble, np.clongdouble]:
+            x = tp(1+2j)
+            assert_warns(np.ComplexWarning, float, x)
+            ctx = WarningManager()
+            ctx.__enter__()
+            warnings.simplefilter('ignore')
+            assert_equal(float(x), float(x.real))
+            ctx.__exit__()
+
+    def test_complex_scalar_complex_cast(self):
+        for tp in [np.csingle, np.cdouble, np.clongdouble]:
+            x = tp(1+2j)
+            assert_equal(complex(x), 1+2j)
+
+    def test_uint_int_conversion(self):
+        x = 2**64 - 1
+        assert_equal(int(np.uint64(x)), x)
+
+    def test_duplicate_field_names_assign(self):
+        ra = np.fromiter(((i*3, i*2) for i in xrange(10)), dtype='i8,f8')
+        ra.dtype.names = ('f1', 'f2')
+        rep = repr(ra) # should not cause a segmentation fault
+        assert_raises(ValueError, setattr, ra.dtype, 'names', ('f1', 'f1'))
+
+    def test_eq_string_and_object_array(self):
+        # From e-mail thread "__eq__ with str and object" (Keith Goodman)
+        a1 = np.array(['a', 'b'], dtype=object)
+        a2 = np.array(['a', 'c'])
+        assert_array_equal(a1 == a2, [True, False])
+        assert_array_equal(a2 == a1, [True, False])
+
+    def test_nonzero_byteswap(self):
+        a = np.array([0x80000000, 0x00000080, 0], dtype=np.uint32)
+        a.dtype = np.float32
+        assert_equal(a.nonzero()[0], [1])
+        a = a.byteswap().newbyteorder()
+        assert_equal(a.nonzero()[0], [1]) # [0] if nonzero() ignores swap
+
+    def test_find_common_type_boolean(self):
+        # Ticket #1695
+        assert_(np.find_common_type([],['?','?']) == '?')
+
+    def test_empty_mul(self):
+        a = np.array([1.])
+        a[1:1] *= 2
+        assert_equal(a, [1.])
+
+    def test_array_side_effect(self):
+        assert_equal(np.dtype('S10').itemsize, 10)
+
+        A = np.array([['abc', 2], ['long   ', '0123456789']], dtype=np.string_)
+
+        # This was throwing an exception because in ctors.c,
+        # discover_itemsize was calling PyObject_Length without checking
+        # the return code.  This failed to get the length of the number 2,
+        # and the exception hung around until something checked
+        # PyErr_Occurred() and returned an error.
+        assert_equal(np.dtype('S10').itemsize, 10)
+
+    def test_any_float(self):
+        # all and any for floats
+        a = np.array([0.1, 0.9])
+        assert_(np.any(a))
+        assert_(np.all(a))
+
+    def test_large_float_sum(self):
+        a = np.arange(10000, dtype='f')
+        assert_equal(a.sum(dtype='d'), a.astype('d').sum())
+
+    def test_ufunc_casting_out(self):
+        a = np.array(1.0, dtype=np.float32)
+        b = np.array(1.0, dtype=np.float64)
+        c = np.array(1.0, dtype=np.float32)
+        np.add(a, b, out=c)
+        assert_equal(c, 2.0)
+
+    def test_array_scalar_contiguous(self):
+        # Array scalars are both C and Fortran contiguous
+        assert_(np.array(1.0).flags.c_contiguous)
+        assert_(np.array(1.0).flags.f_contiguous)
+        assert_(np.array(np.float32(1.0)).flags.c_contiguous)
+        assert_(np.array(np.float32(1.0)).flags.f_contiguous)
+
+    def test_object_array_self_reference(self):
+        # Object arrays with references to themselves can cause problems
+        a = np.array(0, dtype=object)
+        a[()] = a
+        assert_raises(TypeError, int, a)
+        assert_raises(TypeError, long, a)
+        assert_raises(TypeError, float, a)
+        assert_raises(TypeError, oct, a)
+        assert_raises(TypeError, hex, a)
+
+        # This was causing a to become like the above
+        a = np.array(0, dtype=object)
+        a[...] += 1
+        assert_equal(a, 1)
+
+    def test_zerosize_accumulate(self):
+        "Ticket #1733"
+        x = np.array([[42, 0]], dtype=np.uint32)
+        assert_equal(np.add.accumulate(x[:-1,0]), [])
+
+    def test_objectarray_setfield(self):
+        # Setfield directly manipulates the raw array data,
+        # so is invalid for object arrays.
+        x = np.array([1,2,3], dtype=object)
+        assert_raises(RuntimeError, x.setfield, 4, np.int32, 0)
+
+    def test_setting_rank0_string(self):
+        "Ticket #1736"
+        s1 = asbytes("hello1")
+        s2 = asbytes("hello2")
+        a = np.zeros((), dtype="S10")
+        a[()] = s1
+        assert_equal(a, np.array(s1))
+        a[()] = np.array(s2)
+        assert_equal(a, np.array(s2))
+
+        a = np.zeros((), dtype='f4')
+        a[()] = 3
+        assert_equal(a, np.array(3))
+        a[()] = np.array(4)
+        assert_equal(a, np.array(4))
+
+    @dec.knownfailureif(sys.version_info[0] >= 3,
+                        "a.dtype is U5 for Py 3.x. Knownfail for 1.6.x")
+    def test_string_astype(self):
+        "Ticket #1748"
+        s1 = asbytes('black')
+        s2 = asbytes('white')
+        s3 = asbytes('other')
+        a = np.array([[s1],[s2],[s3]])
+        assert_equal(a.dtype, np.dtype('S5'))
+        b = a.astype('str')
+        assert_equal(b.dtype, np.dtype('S5'))
+
+    def test_ticket_1756(self):
+        """Ticket #1756 """
+        s = asbytes('0123456789abcdef')
+        a = np.array([s]*5)
+        for i in range(1,17):
+            a1 = np.array(a, "|S%d"%i)
+            a2 = np.array([s[:i]]*5)
+            assert_equal(a1, a2)
+
+    def test_fields_strides(self):
+        "Ticket #1760"
+        r=np.fromstring('abcdefghijklmnop'*4*3, dtype='i4,(2,3)u2')
+        assert_equal(r[0:3:2]['f1'], r['f1'][0:3:2])
+        assert_equal(r[0:3:2]['f1'][0], r[0:3:2][0]['f1'])
+        assert_equal(r[0:3:2]['f1'][0][()], r[0:3:2][0]['f1'][()])
+        assert_equal(r[0:3:2]['f1'][0].strides, r[0:3:2][0]['f1'].strides)
+
+    def test_ticket_1770(self):
+        "Should not segfault on python 3k"
+        import numpy as np
+        try:
+            a = np.zeros((1,), dtype=[('f1', 'f')])
+            a['f1'] = 1
+            a['f2'] = 1
+        except ValueError:
+            pass
+        except:
+            raise AssertionError
+
+    def test_structured_type_to_object(self):
+        a_rec = np.array([(0,1), (3,2)], dtype='i4,i8')
+        a_obj = np.empty((2,), dtype=object)
+        a_obj[0] = (0,1)
+        a_obj[1] = (3,2)
+        # astype records -> object
+        assert_equal(a_rec.astype(object), a_obj)
+        # '=' records -> object
+        b = np.empty_like(a_obj)
+        b[...] = a_rec
+        assert_equal(b, a_obj)
+        # '=' object -> records
+        b = np.empty_like(a_rec)
+        b[...] = a_obj
+        assert_equal(b, a_rec)
+
+    def test_assign_obj_listoflists(self):
+        # Ticket # 1870
+        # The inner list should get assigned to the object elements
+        a = np.zeros(4, dtype=object)
+        b = a.copy()
+        a[0] = [1]
+        a[1] = [2]
+        a[2] = [3]
+        a[3] = [4]
+        b[...] = [[1], [2], [3], [4]]
+        assert_equal(a, b)
+        # The first dimension should get broadcast
+        a = np.zeros((2,2), dtype=object)
+        a[...] = [[1,2]]
+        assert_equal(a, [[1,2], [1,2]])
+
 if __name__ == "__main__":
     run_module_suite()
