@@ -1,20 +1,20 @@
-import os
 import sys
-
-from numpy.distutils.cpuinfo import cpu
 from numpy.distutils.fcompiler import FCompiler
 
+compilers = ['NAGFCompiler']
+
 class NAGFCompiler(FCompiler):
 
     compiler_type = 'nag'
+    description = 'NAGWare Fortran 95 Compiler'
     version_pattern =  r'NAGWare Fortran 95 compiler Release (?P<version>[^\s]*)'
 
     executables = {
-        'version_cmd'  : ["f95", "-V"],
+        'version_cmd'  : ["<F90>", "-V"],
         'compiler_f77' : ["f95", "-fixed"],
         'compiler_fix' : ["f95", "-fixed"],
         'compiler_f90' : ["f95"],
-        'linker_so'    : ["f95"],
+        'linker_so'    : ["<F90>"],
         'archiver'     : ["ar", "-cr"],
         'ranlib'       : ["ranlib"]
         }
@@ -27,7 +27,7 @@ class NAGFCompiler(FCompiler):
         return ['-O4']
     def get_flags_arch(self):
         version = self.get_version()
-        if version < '5.1':
+        if version and version < '5.1':
             return ['-target=native']
         else:
             return ['']
@@ -40,4 +40,4 @@ if __name__ == '__main__':
     from numpy.distutils.fcompiler import new_fcompiler
     compiler = new_fcompiler(compiler='nag')
     compiler.customize()
-    print compiler.get_version()
+    print(compiler.get_version())
